@@ -2,9 +2,12 @@
 Restore pgdump files from Dropbox.
 See __init__.py for a list of options.
 """
+from __future__ import (absolute_import, division,
+                        print_function, unicode_literals)
 import os
 import tempfile
 import gzip
+import sys
 
 from ... import utils
 from ...dbcommands import DBCommands
@@ -18,12 +21,23 @@ from django.db import connection
 from optparse import make_option
 
 
+# Fix Python 2.x.
+try:
+    input = raw_input  # @ReservedAssignment
+except NameError:
+    pass
+
+
 class Command(LabelCommand):
     help = "dbrestore [-d <dbname>] [-f <filename>] [-s <servername>]"
     option_list = BaseCommand.option_list + (
         make_option("-d", "--database", help="Database to restore"),
         make_option("-f", "--filepath", help="Specific file to backup from"),
+        make_option("-x", "--backup-extension", help="The extension to use when scanning for files to restore from."),
         make_option("-s", "--servername", help="Use a different servername backup"),
+        make_option("-l", "--list", action='store_true', default=False, help="List backups in the backup directory"),
+        make_option("-c", "--decrypt", help="Decrypt data before restoring", default=False, action='store_true'),
+        make_option("-z", "--uncompress", help="Uncompress gzip data before restoring", action='store_true'),
     )
 
     def handle(self, **options):
@@ -31,12 +45,17 @@ class Command(LabelCommand):
         try:
             connection.close()
             self.filepath = options.get('filepath')
+            self.backup_extension = options.get('backup-extension') or 'backup'
             self.servername = options.get('servername')
+            self.decrypt = options.get('decrypt')
+            self.uncompress = options.get('uncompress')
             self.database = self._get_database(options)
             self.storage = BaseStorage.storage_factory()
             self.dbcommands = DBCommands(self.database)
+            if options.get('list'):
+                return self.list_backups()
             self.restore_backup()
-        except StorageError, err:
+        except StorageError as err:
             raise CommandError(err)
 
     def _get_database(self, options):
@@ -47,34 +66,40 @@ class Command(LabelCommand):
                 errmsg = "Because this project contains more than one database, you"
                 errmsg += " must specify the --database option."
                 raise CommandError(errmsg)
-            database_key = settings.DATABASES.keys()[0]
+            database_key = list(settings.DATABASES.keys())[0]
         return settings.DATABASES[database_key]
 
     def restore_backup(self):
         """ Restore the specified database. """
-        print "Restoring backup for database: %s" % self.database['NAME']
+        print("Restoring backup for database: %s" % self.database['NAME'])
         # Fetch the latest backup if filepath not specified
         if not self.filepath:
-            print "  Finding latest backup"
+            print("  Finding latest backup")
             filepaths = self.storage.list_directory()
-            filepaths = self.dbcommands.filter_filepaths(filepaths, self.servername)
+            filepaths = list(filter(lambda f: f.endswith('.' + self.backup_extension), filepaths))
             if not filepaths:
-                raise CommandError("No backup files found in: %s" % self.storage.backup_dir())
+                raise CommandError("No backup files found in: /%s" % self.storage.backup_dir)
             self.filepath = filepaths[-1]
         # Restore the specified filepath backup
-        print "  Restoring: %s" % self.filepath
+        print("  Restoring: %s" % self.filepath)
         input_filename = self.filepath
         inputfile = self.storage.read_file(input_filename)
-        if self.get_extension(input_filename) == '.gpg':
+        if self.decrypt:
+            raise
             unencrypted_file = self.unencrypt_file(inputfile)
             inputfile.close()
             inputfile = unencrypted_file
             input_filename = inputfile.name
-        if self.get_extension(input_filename) == '.gz':
+        if self.uncompress:
             uncompressed_file = self.uncompress_file(inputfile)
             inputfile.close()
             inputfile = uncompressed_file
-        print "  Restore tempfile created: %s" % utils.handle_size(inputfile)
+        print("  Restore tempfile created: %s" % utils.handle_size(inputfile))
+        cont = input("Are you sure you want to continue? [Y/n]")
+        if cont.lower() != 'y':
+            print("Quitting")
+            sys.exit(0)
+        inputfile.seek(0)
         self.dbcommands.run_restore_commands(inputfile)
 
     def get_extension(self, filename):
@@ -83,8 +108,8 @@ class Command(LabelCommand):
 
     def uncompress_file(self, inputfile):
         """ Uncompress this file using gzip. The input and the output are filelike objects. """
-        outputfile = tempfile.SpooledTemporaryFile(max_size=10 * 1024 * 1024)
-        zipfile = gzip.GzipFile(fileobj=inputfile, mode="rb")
+        outputfile = tempfile.SpooledTemporaryFile(max_size=500 * 1024 * 1024)
+        zipfile = gzip.GzipFile(fileobj=inputfile, mode="r")
         try:
             inputfile.seek(0)
             outputfile.write(zipfile.read())
@@ -96,8 +121,8 @@ class Command(LabelCommand):
         """ Unencrypt this file using gpg. The input and the output are filelike objects. """
         import gnupg
         def get_passphrase():
-            print 'Input Passphrase: '
-            return raw_input()
+            print('Input Passphrase: ')
+            return input()
         
         temp_dir = tempfile.mkdtemp()
         try:
@@ -122,4 +147,11 @@ class Command(LabelCommand):
                     os.remove(temp_filename)
         finally:
             os.rmdir(temp_dir)
-        return outputfile
\ No newline at end of file
+        return outputfile
+
+    def list_backups(self):
+        """ List backups in the backup directory. """
+        print("Listing backups on %s in /%s:" % (self.storage.name, self.storage.backup_dir))
+        for filepath in self.storage.list_directory():
+            print("  %s" % os.path.basename(filepath))
+            print(utils.filename_details(filepath))
